//
//  MyDocument.m
//  Validator-SAC
//
//  Created by Chuck Houpt on 3/23/06.
//  Copyright Chuck Houpt 2006 . All rights reserved.
//

#import "MyDocument.h"

NSString *HTMLDocumentType = @"HTML document";

@implementation MyDocument


- (id)init
{
    self = [super init];
    if (self) {
		docURL = nil;
		task = nil;
		standardData = nil;
		errorData = nil;
    }
    return self;
}

- (NSString *)windowNibName
{
    // Override returning the nib file name of the document
    // If you need to use a subclass of NSWindowController or if your document supports multiple NSWindowControllers, you should remove this method and override -makeWindowControllers instead.
    return @"MyDocument";
}

- (NSString *)displayName
{
	return @"Validator S.A.C.";
	/*
	if (docURL != nil) {
		return [docURL absoluteString];
	} else 	return [super displayName]; 
	*/
	//return @"foo";
}

- (BOOL)isDocumentEdited
{
	return NO;
}

- (BOOL)validateMenuItem:(NSMenuItem *)anItem
{
	return NO;
}

-(NSURL *)htdocsURL
{
	NSBundle *thisBundle = [NSBundle bundleForClass:[self class]];

	NSString *validatorPath = [NSString stringWithFormat:@"%@/htdocs-weblet/index.html", [thisBundle pathForResource:@"validator" ofType:@""]];
	
	return [NSURL fileURLWithPath: validatorPath];
}

- (void)loadHTDocName: (NSString *)file
{
		NSURL *indexURL = [NSURL URLWithString:file relativeToURL: [self htdocsURL]];
	    NSURLRequest *request = [[NSURLRequest alloc] initWithURL:indexURL];
		[[webView mainFrame] loadRequest:request];
		[request release];
}

- (void)runWebletFromString:(NSString *)query
{
	NSBundle *thisBundle = [NSBundle bundleForClass:[self class]];

	NSString *scriptPath = [thisBundle pathForResource:@"weblet" ofType:nil];
	NSString *launchPath = @"/bin/bash";
	
	task = [[NSTask alloc] init];
	
	standardData = [[NSMutableData data] retain];
	errorData = [[NSMutableData data] retain];
	
	NSPipe *output = [NSPipe pipe];
	NSPipe *error = [NSPipe pipe];
	
	[task setStandardOutput: output];
    [task setStandardError: error];
    // The path to the binary is the first argument that was passed in
    [task setLaunchPath: launchPath];
	NSArray *arguments = [NSArray arrayWithObjects: scriptPath, query, nil];
	[task setArguments: arguments];

	[self loadHTDocName: @"loading.html"];

    [[NSNotificationCenter defaultCenter] addObserver:self 
        selector:@selector(getStandardData:) 
        name: NSFileHandleReadCompletionNotification 
        object: [[task standardOutput] fileHandleForReading]];	
	[[[task standardOutput] fileHandleForReading] readInBackgroundAndNotify];

    [[NSNotificationCenter defaultCenter] addObserver:self 
        selector:@selector(getErrorData:) 
        name: NSFileHandleReadCompletionNotification 
        object: [[task standardError] fileHandleForReading]];
	[[[task standardError] fileHandleForReading] readInBackgroundAndNotify];

	[[NSNotificationCenter defaultCenter] addObserver:self 
		selector:@selector(webletTerminated:) 
		name: NSTaskDidTerminateNotification 
		object: task];
	
	[task launch];
}

- (void) getStandardData: (NSNotification *)aNotification
{
    NSData *data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];
	if ([data length]) {
		[standardData appendData: data];
		[[aNotification object] readInBackgroundAndNotify]; 
	}
}

- (void) getErrorData: (NSNotification *)aNotification
{
    NSData *data = [[aNotification userInfo] objectForKey:NSFileHandleNotificationDataItem];
	if ([data length]) {
		[standardData appendData: data];
		[[aNotification object] readInBackgroundAndNotify];
	} 
}

- (void)webletTerminated:(NSNotification *)aNotification {
	NSString *mimetype;
	NSData *output;
	
	[[NSNotificationCenter defaultCenter] removeObserver:self name:NSFileHandleReadCompletionNotification object: [[task standardOutput] fileHandleForReading]];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSFileHandleReadCompletionNotification object: [[task standardError] fileHandleForReading]];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:NSTaskDidTerminateNotification object: task];
	
	if ([task terminationStatus] != 0 || [errorData length] ) {
		// Error detected, so generate report
		NSString *m = [NSString stringWithFormat:@"VALIDATOR-SAC ERROR DETECTED: Please send this report to chuck@habilis.net\n\nRESULT CODE: %d\n\nERROR OUTPUT:\n", [task terminationStatus]];
		NSMutableData *erroutput = [NSMutableData dataWithData:[m dataUsingEncoding: NSUTF8StringEncoding]];
		[erroutput appendData: errorData];
		[erroutput appendData: [@"\n\nSTANDARD OUPUT:\n" dataUsingEncoding: NSUTF8StringEncoding]];
		[erroutput appendData: standardData];
		output = erroutput;
		mimetype = @"text/plain";
	} else {
		// No error, so display HTML after headers 
		const char *start = (const char *)[standardData bytes], *html;
		int len = [standardData length];

		html = strnstr(start, "\n\n", len);
		
		if (html) output = [NSMutableData dataWithBytes: html length: len - (html - start)];
		else output = standardData;
		
		mimetype = @"text/html";
	}
	
	[[webView mainFrame] loadData:output MIMEType:mimetype textEncodingName:@"utf-8" baseURL:[self htdocsURL]];
   
	[webView setNeedsDisplay:YES];
	
	[task release];
	[standardData release];
	[errorData release];
	task = nil;
	standardData = nil;
	errorData = nil;

}

- (void)runWebletFromURL:(NSURL *)URL
{
	[self runWebletFromString: [URL absoluteString]];
}

- (void)windowControllerDidLoadNib:(NSWindowController *) aController
{
	windowController = aController;
    [super windowControllerDidLoadNib:aController];
    // Add any code here that needs to be executed once the windowController has loaded the document's window.

	// Hack to avoid problem with Sparkles font setting.
	[webView setPreferencesIdentifier:@"net.habilis.validator-sac.webview"];
	[[webView preferences] setStandardFontFamily:@"Times"];
	[[webView preferences] setDefaultFontSize:16];
	
	if (docURL != nil) {
		//NSData *output = [self runWebletFromURL: initURL];
		//[[webView mainFrame] loadData:output MIMEType:@"text/html" textEncodingName:@"utf-8" baseURL:[self htdocsURL]];
		[self runWebletFromURL: docURL];
		[windowController synchronizeWindowTitleWithDocumentName];
	} else {
		[self loadHTDocName: @"index.html"];

		//[[webView mainFrame] loadHTMLString:@"<html><body>Loading...</body></html>" baseURL:[self htdocsURL]];
	}
}

- (BOOL)readFromFile:(NSString *)fileName ofType:(NSString *)docType
{
	NSURL *URL = [NSURL fileURLWithPath:fileName];
	docURL = URL;
	return TRUE;
}


- (BOOL)readFromURL:(NSURL *)URL ofType:(NSString *)docType error:(NSError **)outError
{
	docURL = URL;
	return TRUE;
}


/*
- (NSData *)dataRepresentationOfType:(NSString *)aType
{
    // Insert code here to write your document from the given data.  You can also choose to override -fileWrapperRepresentationOfType: or -writeToFile:ofType: instead.
    
    // For applications targeted for Tiger or later systems, you should use the new Tiger API -dataOfType:error:.  In this case you can also choose to override -writeToURL:ofType:error:, -fileWrapperOfType:error:, or -writeToURL:ofType:forSaveOperation:originalContentsURL:error: instead.

    return nil;
}

- (BOOL)loadDataRepresentation:(NSData *)data ofType:(NSString *)aType
{
    // Insert code here to read your document from the given data.  You can also choose to override -loadFileWrapperRepresentation:ofType: or -readFromFile:ofType: instead.
    
    // For applications targeted for Tiger or later systems, you should use the new Tiger API readFromData:ofType:error:.  In this case you can also choose to override -readFromURL:ofType:error: or -readFromFileWrapper:ofType:error: instead.
    
    return YES;
}
*/

- (void)goToURL:(NSURL *)URL
{
	docURL = URL;
	[self runWebletFromURL: URL];
}

- (void)webView:(WebView *)view decidePolicyForNavigationAction:(NSDictionary *)actionInformation request:(NSURLRequest *)request frame:(WebFrame *)frame decisionListener:(id<WebPolicyDecisionListener>)listener
{
    int navigationType = [[actionInformation objectForKey:WebActionNavigationTypeKey] intValue];
    if (navigationType == WebNavigationTypeOther) {
        [listener use];
    } else if (navigationType == WebNavigationTypeLinkClicked) {
		NSURL *url = [actionInformation objectForKey:WebActionOriginalURLKey];
		if (! [url isFileURL]) {
			[[NSWorkspace sharedWorkspace] openURL:[actionInformation objectForKey:WebActionOriginalURLKey]];
			[listener ignore];
		} else {
			if ([[url resourceSpecifier] hasSuffix: @"/"]) {
				NSURL *indexURL = [[NSURL alloc] initWithString:@"index.html" relativeToURL:url];
				NSURLRequest *request = [[NSURLRequest alloc] initWithURL:indexURL];
				[[webView mainFrame] loadRequest:request];
				[request release];
				[listener ignore];
			} else {
				[listener use];
			}
		}
	} else if (navigationType == WebNavigationTypeFormSubmitted) {
		NSURL *url = [actionInformation objectForKey:WebActionOriginalURLKey];
		docURL = url;
		[self runWebletFromString: [url query]];

    } else {
        // Ignore WebNavigationTypeFormSubmitted, WebNavigationTypeBackForward, 
        // WebNavigationTypeReload and WebNavigationTypeFormResubmitted.
        [listener ignore];
    }
}


@end
